﻿using EPiServer.Find;
using EPiServer.Find.Cms;
using EPiServer.Find.Framework.Statistics;
using EPiServer.Find.Personalization;
using EPiServer.Reference.Commerce.Site.Features.Personalization.Models;
using EPiServer.Reference.Commerce.Site.Features.Personalization.ViewModels;
using EPiServer.Reference.Commerce.Site.Features.Product.Models;
using EPiServer.Web.Mvc;
using EPiServer.Web.Routing;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;

namespace EPiServer.Reference.Commerce.Site.Features.Personalization.Controllers
{
    public class PersonalizedFindPageController : PageController<PersonalizedFindPage>
    {
        private readonly IClient _client;
        private readonly UrlResolver _urlResolver;

        public PersonalizedFindPageController(IClient client, UrlResolver urlResolver)
        {
            _client = client;
            _urlResolver = urlResolver;
        }

        public ActionResult Index(PersonalizedFindPage currentPage, string q, bool? usePersonalization = null)
        {
            var viewModel = new PersonalizedFindPageViewModel { CurrentPage = currentPage, SearchTerm = q, UsePersonalization = usePersonalization.HasValue && usePersonalization.Value };

            _client.Personalization().Refresh();

            var prefData = _client.Personalization().Conventions.PreferenceRepository.Load();

            if (prefData != null)
            {
                var attributes = prefData.Attributes;
                viewModel.CurrentPersonalizationAttributes = attributes;
            }

            if (!string.IsNullOrEmpty(q))
            {
                var resultsPersonalized = _client.Search<FashionProduct>()
                    .For(q)
                    .FilterForVisitor()
                    .Track()
                    .UsingPersonalization()
                    .GetContentResult();

                var results = _client.Search<FashionProduct>()
                    .For(q)
                    .FilterForVisitor()
                    .Track()
                    .GetContentResult();

                if (results != null)
                {
                    viewModel.SearchResults = GetSearchResultModels(results);
                }
                if (resultsPersonalized != null)
                {
                    viewModel.PersonalizedSearchResults = GetSearchResultModels(resultsPersonalized);
                }
            }

            return View(viewModel);
        }

        private IEnumerable<SearchResultModel> GetSearchResultModels(IContentResult<FashionProduct> contentResult)
        {
            return contentResult.Select(fashionProduct => new SearchResultModel
            {
                Name = fashionProduct.Name,
                Brand = fashionProduct.Brand,
                AvailableColors = fashionProduct.AvailableColors,
                Score = contentResult.SearchResult.Hits.First(x => x.Document.ContentLink == fashionProduct.ContentLink).Score.GetValueOrDefault(),
                Url = _urlResolver.GetUrl(fashionProduct.ContentLink),
            });
        }
    }
}